/************************************************************
* $Id: main.c,v 1.1 2015/11/11 18:16:12 mechanoid Exp mechanoid $
*  
*  OpenCL demo
*
*       OS : GNU/Linux 4.2.3-1-ARCH x86_64 
* COMPILER : GNU project C and C++ Compiler  v.5.2.0
*
*   AUTHOR : Evgeny S. Borisov
*
*    http://www.mechanoid.kiev.ua
*  e-mail : opencl@mechanoid.kiev.ua
* 
************************************************************/

#include <stdio.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#define STR_BUF_SIZE  1024


#define CL_CATCH_ERROR( FF , MSG ) \
   { if( (FF) != CL_SUCCESS) { fprintf(stderr,"[E] "MSG", line: %d\n",__LINE__); exit(-1); } }

#define CATCH_MALLOC_ERROR( FF ) \
   { if( (FF) == NULL ) { fprintf(stderr,"[E] malloc, line: %d\n",__LINE__); exit(-1); } }


#define CL_PRINT_PLATFORM_INFO_STR( PLT , FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetPlatformInfo (PLT, FLD, STR_BUF_SIZE, BUF, NULL) , "clGetPlatformInfo"  ); \
	 fprintf(stdout, MSG" : %s\n", BUF); }

#define CL_PRINT_PLATFORM_INFO_INT( PLT , FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetPlatformInfo (PLT, FLD, STR_BUF_SIZE, BUF, NULL) , "clGetPlatformInfo"  ); \
	 fprintf(stdout, MSG" : %d\n", BUF); }


#define CL_PRINT_DEVICE_INFO_STR( DEV, FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetDeviceInfo( DEV, FLD, STR_BUF_SIZE, BUF, NULL) , "clGetPlatformInfo"  ); \
	 fprintf(stdout,"\t"MSG" : %s\n", BUF); }

#define CL_PRINT_DEVICE_INFO_INT( DEV, FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetDeviceInfo( DEV, FLD, sizeof(BUF), &BUF, NULL) , "clGetPlatformInfo"  ); \
	 fprintf(stdout,"\t"MSG" : %d\n", BUF); }

#define CL_PRINT_DEVICE_INFO_LGC( DEV, FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetDeviceInfo( DEV, FLD, sizeof(BUF), &BUF, NULL) , "clGetPlatformInfo"  ); \
	 fprintf(stdout,"\t"MSG" : %s\n", (BUF)?"yes":"no"); }

#define CL_PRINT_DEVICE_INFO_TYP( DEV,      MSG, BUF ) \
   { CL_CATCH_ERROR( clGetDeviceInfo( DEV, CL_DEVICE_TYPE, sizeof(BUF), &BUF, NULL) , "clGetPlatformInfo"  ); \
	 switch(BUF) { \
		case CL_DEVICE_TYPE_DEFAULT: fprintf(stdout,"\t"MSG" : CL_DEVICE_TYPE_DEFAULT\n");  break; \
		case CL_DEVICE_TYPE_CPU: fprintf(stdout,"\t"MSG" : CL_DEVICE_TYPE_CPU\n"); break; \
		case CL_DEVICE_TYPE_GPU: fprintf(stdout,"\t"MSG" : CL_DEVICE_TYPE_GPU\n"); break; \
		case CL_DEVICE_TYPE_ACCELERATOR: fprintf(stdout,"\t"MSG" : CL_DEVICE_TYPE_ACCELERATOR\n"); break; \
		case CL_DEVICE_TYPE_CUSTOM: fprintf(stdout,"\t"MSG" : CL_DEVICE_TYPE_CUSTOM\n"); break; \
		default: fprintf(stdout,"\t"MSG" : unknown\n"); }; }

#define CL_PRINT_DEVICE_INFO_MEM( DEV, FLD, MSG, BUF ) \
   { CL_CATCH_ERROR( clGetDeviceInfo( DEV, FLD, sizeof(BUF), &BUF, NULL) , "clGetPlatformInfo"  ); \
	 if(BUF<1023) fprintf(stdout,"\t"MSG" : %i B\n", BUF ); \
	 else if(BUF<1048576) fprintf(stdout,"\t"MSG" : %.1f KB\n", (double)BUF/ 1024.0 ); \
	 else if(BUF<1073741824) fprintf(stdout,"\t"MSG" : %.1f MB\n", (double)BUF/ 1048576.0 ); \
	 else fprintf(stdout,"\t"MSG" : %.1f GB\n", (double)BUF/ 1073741824.0 ); }




// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void print_platform_info(cl_platform_id platform) {
    char queryBuffer[STR_BUF_SIZE];

	CL_PRINT_PLATFORM_INFO_STR( platform, CL_PLATFORM_NAME, "CL_PLATFORM_NAME", queryBuffer );
	CL_PRINT_PLATFORM_INFO_STR( platform, CL_PLATFORM_VENDOR, "CL_PLATFORM_VENDOR", queryBuffer );
	CL_PRINT_PLATFORM_INFO_STR( platform, CL_PLATFORM_VERSION, "CL_PLATFORM_VERSION", queryBuffer );
	CL_PRINT_PLATFORM_INFO_STR( platform, CL_PLATFORM_PROFILE, "CL_PLATFORM_PROFILE", queryBuffer );
	CL_PRINT_PLATFORM_INFO_STR( platform, CL_PLATFORM_EXTENSIONS, "CL_PLATFORM_EXTENSIONS", queryBuffer );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void print_device_info(cl_device_id device) {
   char queryBuffer[STR_BUF_SIZE];
   cl_device_type dtype;
   cl_uint queryInt;
   cl_ulong queryLong;
   size_t *wsz=NULL;
   int i;

   CL_PRINT_DEVICE_INFO_STR( device, CL_DEVICE_NAME, "CL_DEVICE_NAME", queryBuffer );
   CL_PRINT_DEVICE_INFO_STR( device, CL_DEVICE_VENDOR, "CL_DEVICE_VENDOR", queryBuffer );
   CL_PRINT_DEVICE_INFO_TYP( device, "CL_DEVICE_TYPE", dtype );
   CL_PRINT_DEVICE_INFO_LGC( device, CL_DEVICE_AVAILABLE, "CL_DEVICE_AVAILABLE", queryInt);
   CL_PRINT_DEVICE_INFO_LGC( device, CL_DEVICE_COMPILER_AVAILABLE, "CL_DEVICE_COMPILER_AVAILABLE", queryInt);
   CL_PRINT_DEVICE_INFO_STR( device, CL_DEVICE_VERSION, "CL_DEVICE_VERSION", queryBuffer );
   CL_PRINT_DEVICE_INFO_STR( device, CL_DEVICE_PROFILE, "CL_DEVICE_PROFILE", queryBuffer );
   CL_PRINT_DEVICE_INFO_LGC( device, CL_DEVICE_IMAGE_SUPPORT, "CL_DEVICE_IMAGE_SUPPORT", queryInt);
   CL_PRINT_DEVICE_INFO_INT( device, CL_DEVICE_MAX_COMPUTE_UNITS, "CL_DEVICE_MAX_COMPUTE_UNITS", queryInt);

   CL_PRINT_DEVICE_INFO_INT( device, CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS, "CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS", queryInt);
   
   CATCH_MALLOC_ERROR( wsz=(size_t*)malloc(sizeof(size_t) * queryInt ) );
   CL_CATCH_ERROR( clGetDeviceInfo( device, CL_DEVICE_MAX_WORK_ITEM_SIZES, sizeof(size_t)*queryInt, wsz, NULL) , "clGetPlatformInfo"  ); 
   fprintf(stdout,"\tCL_DEVICE_MAX_WORK_ITEM_SIZES: [");
   for(i=0;i<queryInt;i++){ fprintf(stdout," %d ",wsz[i]); }
   fprintf(stdout,"]\n");
   free(wsz);

   CL_PRINT_DEVICE_INFO_STR( device, CL_DRIVER_VERSION, "CL_DRIVER_VERSION", queryBuffer );
   CL_PRINT_DEVICE_INFO_MEM( device, CL_DEVICE_GLOBAL_MEM_SIZE, "CL_DEVICE_GLOBAL_MEM_SIZE", queryLong);
   CL_PRINT_DEVICE_INFO_MEM( device, CL_DEVICE_LOCAL_MEM_SIZE , "CL_DEVICE_LOCAL_MEM_SIZE", queryLong);
   CL_PRINT_DEVICE_INFO_INT( device, CL_DEVICE_MAX_CLOCK_FREQUENCY, "CL_DEVICE_MAX_CLOCK_FREQUENCY", queryInt);
   CL_PRINT_DEVICE_INFO_STR( device, CL_DEVICE_EXTENSIONS, "CL_DEVICE_EXTENSIONS", queryBuffer );


}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
int main() {
   cl_int           clError;
   cl_platform_id  *platforms;
   cl_device_id    *devices;
   cl_uint          num_platforms, num_devices;
   cl_uint          i,j;

   CL_CATCH_ERROR( clGetPlatformIDs(0, NULL, &num_platforms), "clGetPlatformIDs" );

   if (num_platforms <1 ) { printf("No OpenCL Platforms Found"); return 1; }

   printf ("Found %d Platforms\n\n", num_platforms);

   CATCH_MALLOC_ERROR( platforms = (cl_platform_id *)malloc(num_platforms*sizeof(cl_platform_id)) );
    
   CL_CATCH_ERROR( clGetPlatformIDs (num_platforms, platforms, NULL) , "clGetPlatformIDs" );

   for( i=0 ; i<num_platforms ; i++ ) {
	  printf("==================Platform No %d======================\n",i);
	  print_platform_info(platforms[i]);
   
	  CL_CATCH_ERROR( clGetDeviceIDs (platforms[i], CL_DEVICE_TYPE_ALL, 0, NULL, &num_devices) ,"clGetDeviceIDs" );
	  CATCH_MALLOC_ERROR( devices=(cl_device_id *)malloc(sizeof(cl_device_id) * num_devices) );
	  CL_CATCH_ERROR( clGetDeviceIDs (platforms[i], CL_DEVICE_TYPE_ALL, num_devices, devices, &num_devices) , "clGetDeviceIDs" );

	  printf("\n");
	  for ( j = 0; j < num_devices; j++) {
		 printf("\t------------------Device No %d----------------------\n",j);
		 print_device_info(devices[j]);
		 printf("\n");
	  }

	  free(devices);
	  printf("\n");
   }


   free(platforms);
 
   return 0;
}


